/**
 * Using this template, your task is to implement functions denoted by TODO in the comments.
 * 
 * You do not have to understand the whole source code to finish this task.
 * It is necessary to take a look on the attributes of the class Main
 * (e.g., <code>array</code>).
 * 
 * The Java project also contains tests, in Eclipse you can run them by left-clicking:
 * test/(default package)/JUnitTest.java and selecting Run as -> JUnitTest.java.
 * If you are struggling, try to solve the task for testTwo.
 * 
 * The tests will be always failing in this extra task, as the number of swaps
 * your solution uses can vary. We print the number of swaps, so you can compare
 * with the reference solution, the reference numbers are:
 * <testName>: <number of swaps>/<length of the array>=<ratio of these two>
 * 
 * testTwo: 5/10=0.5
 * testLarge: 567/1000=0.567
 * testSmall: 4/3=1.3333333333333333
 * testSorted: 1/5=0.2
 * testBigger: 7/10=0.7
 * 
 * The reference solution is not optimal, can you beat it?
 * 
 * Few more questions challenging questions if you managed to solve it
 * (although they might be a bit misleading before you solve the problem):
 * 1. Can this be problem be solved for 4 unique numbers, 5, k?
 * 2. What is the complexity given n and k?
 * 3. Would this be an efficient sorting algorithm for general arrays?
 */


//You are forbidden to add any other imports
import java.util.StringTokenizer;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintStream;

class Main {
	private int[] array; // sorted array of integers as input

	/**
	 * Swaps items of <code>array</code> on positions <code>i</code> and <code>j</code>.
	 * It also counts the number of instructions. Do not modify this method.
	 */
	private void swap(int i, int j) {
		int tmp = array[i];
		array[i] = array[j];
		array[j] = tmp;

		instruction_counter += 1;
	}

	/**
	 * TODO: you are given an <code>array</code> containing only numbers 1, 2, and 3.
	 * You are required to sort this array in ascending order with three restrictions:
	 * 
	 * 1. You have to move the numbers in the array using predefined
	 * <code>swap</code> function. I.e., you cannot just count number of occurrences
	 * and then rewrite the content of the array. This is reasonable restriction for
	 * many cases when you operate with more complex objects then just numbers -
	 * sorting people into three groups, when each person has also other attributes
	 * than just a number. 
	 * 2. You have to do the task in place, without allocating any extra array. 
	 * 3. Your code has to run in O(n), where n is the length of the array.
	 * 
	 * More precisely, we measure the number of <code>swap</code> calls, and it has
	 * to be less than 1.2*n. For reference, in average it is possible to use
	 * roughly 0.5*n <code>swap</code> calls.
	 */
	private void tri_sort() {
		int ones = 0;
		int twos = -1;
		int threes = array.length;

		while (twos < threes) {
			while (array[++twos] == 2) {
				if (twos == threes || twos + 1 == array.length)
					return;
			}

			if (array[twos] == 1) {
				swap(ones, twos);
				++ones;
			} else if (array[twos] == 3) {
				while (array[--threes] == 3) {
					if (twos + 1 >= threes)
						return;
				}
				if (array[threes] == 2) {
					swap(threes, twos);
				} else {
					swap(threes, twos);
					swap(twos, ones);
					++ones;

				}
			}
		}
		if (twos > threes) {
			swap(twos, threes);
		}
	}
	
	///////////////////////////////////////////////////////////////////////
	// DO NOT MODIFY THE FOLLOWING CODE, YOU CAN COMPLETELY IGNORE IT
	///////////////////////////////////////////////////////////////////////

	/**
	 * Parses input in form: <length of input array = array_len> <array_len elements
	 * of the array, separated by space>
	 */
	private void read(InputStream in) {
		FastReader s = new FastReader(in);
		int array_len = s.nextInt();
		array = new int[array_len];

		for (int idx = 0; idx < array_len; idx++) {
			array[idx] = s.nextInt();
		}
	}

	/**
	 * Outputs single index per line: <elements of the sorted array, separated by
	 * space> <number of swap calls divided by input array length>
	 * 
	 * @param out   stream where to write
	 * @param index position of the found value, -1 if not found
	 */
	private void output(PrintStream out) {
		for (int x : array) {
			out.print(x + " ");
		}
		out.println("");
		out.println(
				instruction_counter + "/" + array.length + "=" + (double) instruction_counter / (double) array.length);
	}

	/**
	 * Parse the input and for each value, call the binary search.
	 */
	public void read_and_solve(InputStream in, PrintStream out) {
		read(in);

		tri_sort();

		output(out);
	}

	public void main(String[] args) {
		read_and_solve(System.in, System.out);
	}

	/**
	 * Ignore this class please. It is used for input parsing. Source:
	 * https://www.geeksforgeeks.org/fast-io-in-java-in-competitive-programming/
	 */
	static class FastReader {
		BufferedReader br;
		StringTokenizer st;

		public FastReader(InputStream in) {
			br = new BufferedReader(new InputStreamReader(in));
		}

		String next() {
			while (st == null || !st.hasMoreElements()) {
				try {
					st = new StringTokenizer(br.readLine());
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			return st.nextToken();
		}

		int nextInt() {
			return Integer.parseInt(next());
		}

		long nextLong() {
			return Long.parseLong(next());
		}

		double nextDouble() {
			return Double.parseDouble(next());
		}

		String nextLine() {
			String str = "";
			try {
				str = br.readLine();
			} catch (IOException e) {
				e.printStackTrace();
			}
			return str;
		}
	}

	private int instruction_counter; // do not modify content of this variable
}
